unit DynaUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Grids, DBGrids, Db, SQLServer, DBCtrls, ExtCtrls, ComCtrls,
  StdCtrls, Dialogs;
                             
type
  TForm1 = class(TForm)  
    SQLServer1: TSQLServer;
    DataSource1: TDataSource;
    Panel1: TPanel;
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    StatusBar1: TStatusBar;
    Edit1: TEdit;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Button5: TButton;
    SQLQuery1: TSQLQuery;
    procedure FormShow(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure SQLQuery1PostError(DataSet: TDataSet; E: EDatabaseError;
      var Action: TDataAction);
    procedure Button5Click(Sender: TObject);
    procedure DBGrid1TitleClick(Column: TColumn);
    procedure SQLQuery1DuplicateKey(DataSet: TDataSet; E: EDatabaseError;
      var Action: TDataAction);
    procedure SQLQuery1UpdateError(DataSet: TDataSet);
  private
    procedure IdleHandler(Sender: TObject; var Done: Boolean);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormShow(Sender: TObject);
begin
     with SQLServer1 do begin
          DataBaseName := 'pubs';
          UserName     := 'guest';
          ServerName   := 'THOMAS';
          Password     := '';
     end;
     SQLQuery1.CacheMaxRecords := 5000;
     SQLQuery1.Open;
     Application.OnIdle := IdleHandler;
end;
//
// This Idle-handler demonstrates the ReadAhead-method.
// Note that the CacheMaxRecords property maximizes the total number of
// rows to be locally cached.
//
procedure TForm1.IdleHandler(Sender: TObject; var Done: Boolean);
begin
     SQLQuery1.ReadAhead;
     if not SQLQuery1.CacheFull then begin
        Done := False;
        SQLServer1.DBTrace(True);
        Statusbar1.SimpleText := IntToStr(SQLQuery1.RecordCount div 2)+' records'
     end else
        Statusbar1.SimpleText := IntToStr(SQLQuery1.RecordCount)+' records';
// Note: The RecordCount is only correct when reaching EOF (or cache full)
// All other times the RecordCount is the (number of rows read so far) * 2
// This way a DBGrid VerticalScrollbar will always work.
end;


procedure TForm1.Button1Click(Sender: TObject);
begin
     SQLServer1.StartTransaction;
     Button1.Enabled := False;
     Button2.Enabled := True;
     Button3.Enabled := True;
end;

procedure TForm1.Edit1Change(Sender: TObject);
begin
     SQLQuery1.Locate('Company',Edit1.text,[loCaseInsensitive, loPartialKey]);
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
     SQLServer1.Commit;
     Button1.Enabled := True;
     Button2.Enabled := False;
     Button3.Enabled := False;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
     SQLServer1.RollBack;
     Button4Click(Sender);
     Button1.Enabled := True;
     Button2.Enabled := False;
     Button3.Enabled := False;
end;

procedure TForm1.Button4Click(Sender: TObject);
var Temp : TBookMark;
begin
     with SQLQuery1 do begin
          Temp := GetBookMark;
          DisableControls;
          Close;
          Open;
          EnableControls;
          GotoBookMark(Temp);
          FreeBookMark(Temp);
     end;
end;

procedure TForm1.SQLQuery1PostError(DataSet: TDataSet; E: EDatabaseError;
  var Action: TDataAction);
begin   
//   The error codes can be found in the SQL Server manuals, 2601=duplicate key
//   Note that 2601 is will call OnDuplicateKey event handler!
     ShowMessage('ErrorCode='+IntToStr(EcsSQLError(E).ErrorCode));
     Action := daAbort;
end;

procedure TForm1.Button5Click(Sender: TObject);
begin
     SQLQuery1.Append;
end;

procedure TForm1.DBGrid1TitleClick(Column: TColumn);
begin
//   An attempt to do a SortField on a partial result set will raise an error
     if SQLQuery1.CacheFull then SQLQuery1.SortField(Column.FieldName, soAscending);
end;

procedure TForm1.SQLQuery1DuplicateKey(DataSet: TDataSet;
  E: EDatabaseError; var Action: TDataAction);
begin
        ShowMessage('Another customer already assigned to this customer number');
        Action := daAbort;
end;

procedure TForm1.SQLQuery1UpdateError(DataSet: TDataSet);
begin
     ShowMessage('Data changed by another user!');
end;

end.
